package com.ywt.biz.common.config.dubbo;

import org.apache.dubbo.config.ApplicationConfig;
import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.config.RegistryConfig;
import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Component
@ConditionalOnProperty(name = "switch.dubbo.generic", havingValue = "true")
public class GenericServiceFactory {

    @Value("${dubbo.application.name:genericService}")
    private String applicationName;

    @Value("${dubbo.registry.address:nacos://127.0.0.1:8848}")
    private String registryAddress;

    @Value("${dubbo.registry.namespace:public}")
    private String namespace;

    private final Map<String, GenericService> genericServices = new ConcurrentHashMap<>();

    public GenericService getGenericService(String interfaceName) {
        return genericServices.computeIfAbsent(interfaceName, this::createGenericService);
    }

    private GenericService createGenericService(String interfaceName) {
        ApplicationConfig applicationConfig = new ApplicationConfig(applicationName);
        // 配置 Nacos 注册中心和命名空间
        RegistryConfig registryConfig = new RegistryConfig();
        registryConfig.setAddress(registryAddress);
        registryConfig.setProtocol("nacos");
        Map<String, String> parameters  = new HashMap<>();
        parameters.put("namespace",namespace);
        // 可根据实际诉求进行扩展
        registryConfig.setParameters(parameters);

        ReferenceConfig<GenericService> referenceConfig = new ReferenceConfig<>();
        referenceConfig.setApplication(applicationConfig);
        referenceConfig.setRegistry(registryConfig);
        referenceConfig.setInterface(interfaceName);
        referenceConfig.setGeneric("true");
        referenceConfig.setProtocol("tri");

        return referenceConfig.get();
    }
}
