
package com.ywt.biz.common.config.redission.properties;


import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.config.Config;
import org.redisson.config.SentinelServersConfig;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.StringUtils;


/**
 * redisson 配置
 * @author liyang
 */

@Configuration
@EnableConfigurationProperties(RedissonProperties.class)
@ConditionalOnProperty(name = "redisson.enable", havingValue = "true")
public class RedissonConfig {


    @Bean
    public RedissonClient getRedisson(RedissonProperties redissonProperties) {
        Config config = new Config();
        if("single".equals(redissonProperties.getModel())){
            // 单机节点
            config.useSingleServer()
                    .setAddress("redis://" + redissonProperties.getHost() + ":" + redissonProperties.getPort())
                    .setPassword(redissonProperties.getPassword())
                    .setDatabase(redissonProperties.getDatabase())
                    .setRetryInterval(5000)
                    .setTimeout(10000)
                    .setConnectTimeout(10000);
            return Redisson.create(config);
        }
        if("sentinel".equals(redissonProperties.getModel())){
            String[] sentinelAddresses  = redissonProperties.getSentinelAddresses();
            String schema = redissonProperties.getSchema();
            // 拼接协议
            String[] sentinelAddressesWithSchema = new String[sentinelAddresses.length];
            for (int i = 0; i < sentinelAddresses.length; i++) {
                sentinelAddressesWithSchema[i] = schema + sentinelAddresses[i];
            }

            SentinelServersConfig serversConfig = config.useSentinelServers()
                    .setMasterName(redissonProperties.getMasterName())
                    .addSentinelAddress(sentinelAddressesWithSchema)
                    .setMasterConnectionPoolSize(redissonProperties.getMasterConnectionPoolSize())
                    .setSlaveConnectionPoolSize(redissonProperties.getSlaveConnectionPoolSize())
                    .setMasterConnectionMinimumIdleSize(redissonProperties.getConnectionMinimumIdleSize())
                    .setSlaveConnectionMinimumIdleSize(redissonProperties.getConnectionMinimumIdleSize());

            if (StringUtils.hasText(redissonProperties.getPassword())) {
                serversConfig.setPassword(redissonProperties.getPassword());
            }
            return Redisson.create(config);
        }
        return null ;
    }

}

