package com.ywt.biz.common.constant;
/**
 * 响应码常量定义,所有工程均再次定义各自的前缀
 * @author fhp
 *
 */
public interface RespCodePrefix {

	//////////////////////////////////////////////////////////////////////////////////////////////////////
	// 所有业务响应码前缀定义
	//////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * 全局统一正常响应码前缀
	 */
	String PREFIX_COMMON = "100";
	/**
	 * BIZ-COMMON响应码前缀
	 */
	String PREFIX_BIZ_COMMON = "101";
	/**
	 * GATEWAY响应码前缀
	 */
	String PREFIX_GATEWAY = "107";
	/**
	 * 处方响应码前缀
	 */
	String PREFIX_PRESCRIPTION = "108";
	/**
	 * PAYMENT响应码前缀
	 */
	String PREFIX_PAYMENT = "109";
	/**
	 * ORDER响应码前缀
	 */
	String PREFIX_ORDER = "110";


	//////////////////////////////////////////////////////////////////////////////////////////////////////
	// 所有响应码类型定义
	//////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * 正常类型响应码前缀
	 */
	String PREFIX_N = "00";
	/**
	 * 参数类型响应码前缀
	 */
	String PREFIX_P = "10";
	/**
	 * 数据类型响应码前缀
	 */
	String PREFIX_D = "20";
	/**
	 * 服务类型响应码前缀
	 */
	String PREFIX_S = "30";
	/**
	 * 资源类型响应码前缀
	 */
	String PREFIX_R = "40";

	//////////////////////////////////////////////////////////////////////////////////////////////////////
	// 特定响应码定义
	//////////////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * 最大响应码,所有响应码不能超过此值(保证不超过Integer.MAX_VALUE=2147483647)
	 */
	String MAX_RESPONSE_CODE = "2109999999";
	/**
	 * 请求应答未知异常-后5位
	 */
	String RESPONSE_UNKNOWN = "99999";
	/**
	 * 请求应答正常-后5位
	 */
	String RESPONSE_SUCESS = "00000";

	/**
	 * 全局统一正常类型响应码为1000000000
	 */
	String NORMAL_RESP = PREFIX_COMMON+PREFIX_N+RESPONSE_SUCESS;

	//////////////////////////////////////////////////////////////////////////////////////////////////////
	// BIZ-COMMON响应码前两段,正常响应码为全部
	//////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * BIZ-COMMON正常类型响应码为全部
	 */
	String PREFIX_BIZ_COMMON_N = PREFIX_COMMON+PREFIX_N+RESPONSE_SUCESS;
	/**
	 * BIZ-COMMON参数类型响应码前两段
	 */
	String PREFIX_BIZ_COMMON_P = PREFIX_BIZ_COMMON+PREFIX_P;
	/**
	 * BIZ-COMMON数据类型响应码前两段
	 */
	String PREFIX_BIZ_COMMON_D = PREFIX_BIZ_COMMON+PREFIX_D;
	/**
	 * BIZ-COMMON服务类型响应码前两段
	 */
	String PREFIX_BIZ_COMMON_S = PREFIX_BIZ_COMMON+PREFIX_S;
	/**
	 * BIZ-COMMON资源类型响应码前两段
	 */
	String PREFIX_BIZ_COMMON_R = PREFIX_BIZ_COMMON+PREFIX_R;


	//////////////////////////////////////////////////////////////////////////////////////////////////////
	// ORDER响应码前两段,正常响应码为全部
	//////////////////////////////////////////////////////////////////////////////////////////////////////
	String PREFIX_NUMBER_N = PREFIX_COMMON+PREFIX_N+RESPONSE_SUCESS;
	String PREFIX_NUMBER_P = PREFIX_ORDER+PREFIX_P;
	String PREFIX_NUMBER_D = PREFIX_ORDER+PREFIX_D;
	String PREFIX_NUMBER_S = PREFIX_ORDER+PREFIX_S;
	String PREFIX_NUMBER_R = PREFIX_ORDER+PREFIX_R;

	/**
	 * 返回状态码
	 * @return
	 */
	String getCode();

	/**
	 * 返回消息
	 * @return
	 */
	String getMsg();

	/**
	 * 返回明细描述
	 * @return
	 */
	String getDetail();
}