package com.ywt.biz.common.constant;

/**
 * BIZ-COMMON工程专有响应码枚举类
 * 作为自定义异常对象的入参,为使用自定义异常必须针对异常进系枚举定义.这里的XXX/YYY均需要替换成各个真实的变量.
 * 变量的命名规则:保证简介无歧义,不可过长(>10)也不能太短导致无法辨识含义
 * 
 *
 * @author fhp
 *
 */
public enum YwtCommonRespCode implements RespCodePrefix {
	/**
	 * 正常响应码枚举定义
	 */
	REQ_SUCCESS(PREFIX_BIZ_COMMON_N, "响应正常", "response success"),
	/**
	 * 参数类型响应码枚举定义
	 */
	P_ERR(PREFIX_BIZ_COMMON_P + "10000", "参数异常", "parameter exception"),
	P_CLASS_CAST_ERR(PREFIX_BIZ_COMMON_P + "10001", "对象类型转换异常", "class cast exception"),

	/**
	 * 数据类型响应码枚举定义
	 */
	D_ERR(PREFIX_BIZ_COMMON_D + "20000", "数据异常", "data exception"),
	D_DUPLICATEKEY(PREFIX_BIZ_COMMON_D + "20001", "违反唯一约束", ""),

	U_TYPE_NOT_SUPPORTED(PREFIX_BIZ_COMMON_S + "30007", "不支持的枚举类型", ""),

	/**
	 * 物理资源类型响应码枚举定义
	 */
	R_DISK_FULL(PREFIX_BIZ_COMMON_R + "40001", "磁盘空间满", ""),
	R_OFFLINE(PREFIX_BIZ_COMMON_R + "40002", "网络不通", ""),
	R_PORT_IN_USER(PREFIX_BIZ_COMMON_R + "40003", "端口被占用", ""),

	/**
	 * 未知异常枚举定义
	 */
	UNKONWN_ERR(MAX_RESPONSE_CODE+RESPONSE_UNKNOWN, "其他错误", "");

	/**
	 * 响应码编码,格式为 3位业务编码+2位响应码类型编码+5位具体响应码
	 */
	private String code;
	/**
	 * 中文通用描述
	 */
	private String msg;
	/**
	 * 需要提供具体数据时使用.
	 * 方便后续分析查看.
	 * 可以在抛出异常处通过对此属性进行赋值记录现场
	 */
	private String detail;
	YwtCommonRespCode(String code, String msg, String detail ) {
		this.code = code;
		this.msg = msg;
		this.detail = detail;
	}

	@Override
	public String getCode() {
		return code;
	}

	@Override
	public String getMsg() {
		return msg;
	}

	@Override
	public String getDetail() {
		return detail;
	}

}
