package com.ywt.biz.common.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

public class DateUtil {
    public static final String DATE_FORMAT_YMD = "yyyy-MM-dd";

    private static final String[] WEEK_DAYS = {"星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六"};

    private static final String[] WEEK_DAYS_SHORT = {"日", "一", "二", "三", "四", "五", "六"};

    private static final String[] WEEK_DAYS_MIDDLE = {"周日", "周一", "周二", "周三", "周四", "周五", "周六"};

    public static final String DADE_FROMAT_YMD = "yyyy-MM-dd";

    public static final String DADE_FROMAT_YMDHMS = "yyyy-MM-dd HH:mm:ss";

    public static final String DADE_FROMAT_CYMD = "yyyy年M月d日";

    public static final String DADE_FROMAT_CYMDHM = "yyyy年M月d日 HH:mm";



    /**
     * 将时间戳转换成日期字符串格式
     * @param timestamp 时间戳
     * @param pattern 日期格式
     * @return
     */
    public static String convertTimestampToDateString(long timestamp, String pattern) {
        Date date = new Date(timestamp);
        return convertToString(date, pattern);
    }


    public static String convertToString(Date date, String pattern){
        if(StringHelper.isNullOrEmpty(pattern)){
            pattern = "yyyy-MM-dd HH:mm:ss";
        }

        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }

    /**
     * 计算当前时间和指定时间之间相差的小时
     *
     * @param smdate
     *            指定的时间
     * @throws ParseException
     */
    public static int hourBetween(String smdate) {
        Date start = new Date();
        Date end = stringToDate(smdate);
        Calendar cal = Calendar.getInstance();
        cal.setTime(start);
        long time1 = cal.getTimeInMillis();
        cal.setTime(end);
        long time2 = cal.getTimeInMillis();

        long between_days = (time2 - time1) / (1000 * 3600);

        return Integer.parseInt(String.valueOf(between_days));
    }

    /**
     * String转Date
     *
     * @param strDate
     * @return
     */
    public static Date stringToDate(String strDate, String pattern) {
        if (null == strDate) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        try {
            return sdf.parse(strDate);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static Date stringToDate(String strDate) {
        return stringToDate(strDate, "yyyy-MM-dd HH:mm:ss");
    }


    public static String convertToString(Date date){
        return convertToString(date, null);
    }

    /**
     * 当前日期是星期几
     * @param calendar
     * @return
     */
    public static String getDayOfWeek(Calendar calendar) {
        int index = calendar.get(Calendar.DAY_OF_WEEK) - 1;
        if (index < 0) {
            index = 0;
        }
        return WEEK_DAYS[index];
    }

    /**
     * 当前日期是星期（缩略）
     * @param calendar
     * @return
     */
    public static String getDayOfWeekShort(Calendar calendar) {
        int index = calendar.get(Calendar.DAY_OF_WEEK) - 1;
        if (index < 0) {
            index = 0;
        }
        return WEEK_DAYS_SHORT[index];
    }

    /**
     * 当前日期是星期（缩略）
     * @param calendar
     * @return
     */
    public static String getDayOfWeekMiddle(Calendar calendar) {
        int index = calendar.get(Calendar.DAY_OF_WEEK) - 1;
        if (index < 0) {
            index = 0;
        }
        return WEEK_DAYS_MIDDLE[index];
    }

    public static Date parseDate(String str, String pattern) {
        if (StringHelper.isNullOrWhiteSpace(str)) {
            return null;
        }

        if (StringHelper.isNullOrWhiteSpace(pattern)) {
            pattern = "yyyy-MM-dd HH:mm:ss";    //默认日期格式
        }

        SimpleDateFormat format = new SimpleDateFormat(pattern);
        try {
            format.setLenient(false);
            return format.parse(str);
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 得到修改之后的日期
     * @param type:1-天，2-月，3-年
     * @param changeNumber : 往前或往后变动日期，changeNumber < 0 表示往前修改(eg:2019->2018) changeNumber 日/月/年，
     *                     changeNumber > 0 表示往后修改(eg:2018->2019</>) changeNumber 日/月/年，
     * @param date 日期
     * @return 修改之后的日期，格式：yyyy-MM-dd HH:mm:ss
     */
    public static Date getChangeMonthDate(Date date, int type, int changeNumber) throws Exception {
        //获得实体类
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        //设置最后一天
        if (type == 1){
            ca.add(Calendar.DAY_OF_YEAR, changeNumber);
        }else if(type == 2){
            ca.add(Calendar.MONTH, changeNumber);
        }else if (type == 3){
            ca.add(Calendar.YEAR, changeNumber);
        }
        //最后一天格式化
        return ca.getTime();
    }

    public static Date getThreeDayBeforeDate()throws Exception{
        //把日期往后增加一天.整数往后推,负数往前移动
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(new Date());
        //这个时间就是日期往前推3天
        calendar.add(Calendar.DATE, -3);
        return calendar.getTime();
    }

    /**
     * 格式化日期
     *
     * @param date
     * @param pattern
     * @return
     */
    public static String formatDate(Date date, String pattern) {
        if (StringHelper.isNullOrEmpty(pattern)) {
            pattern = "yyyy-MM-dd HH:mm:ss";
        }

        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }

    /**
     * 得到相对时间
     *
     * @param num 天数，大于0表示几天之后的时间，小于0表示几天之前的时间
     * @return 时间
     */
    public static Date getRelativeDate(int num) {
        return getRelativeDate(new Date(), num);
    }

    /**
     * 得到相对时间
     *
     * @param date 传入的时间
     * @param num  前几天，或者后几天
     * @return 改变之后的时间
     */
    public static Date getRelativeDate(Date date, int num) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, num);
        return calendar.getTime();
    }

    /**
     * 将时间戳转换成日期字符串格式
     *
     * @param timestamp 时间戳
     * @return
     */
    public static String convertTimestampToDateString(long timestamp) {
        Date date = new Date(timestamp);
        SimpleDateFormat sdf = new SimpleDateFormat(DADE_FROMAT_YMDHMS);
        return sdf.format(date);
    }

    public static LocalDate convertToLocalDate(Date date) {
        Instant instant = date.toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        return instant.atZone(zoneId).toLocalDate();
    }

    /**
     * 将时间戮转换成 LocalDate 类型
     *
     * @param timestamp 时间戮
     * @return 日期类型 LocalDate
     */
    public static LocalDate convertToLocalDate(long timestamp) {
        return Instant.ofEpochMilli(timestamp).atZone(ZoneId.systemDefault()).toLocalDate();
    }


    /**
     * 得到距离当前的天数
     *
     * @param beginDateStr 开始日期
     * @return 天数
     */
    public static long getCurrentDay(String beginDateStr) {
        return getDaySub(beginDateStr, new SimpleDateFormat(DADE_FROMAT_YMD).format(new Date()));
    }

    /**
     * 得到相减的天数
     *
     * @param beginDateStr 开始时间
     * @param endDateStr   截止时间
     * @return 天数
     */
    public static long getDaySub(String beginDateStr, String endDateStr) {
        long day = 0;
        java.text.SimpleDateFormat format = new java.text.SimpleDateFormat(DADE_FROMAT_YMD);
        java.util.Date beginDate;
        java.util.Date endDate;
        try {
            beginDate = format.parse(beginDateStr);
            endDate = format.parse(endDateStr);
            day = (endDate.getTime() - beginDate.getTime()) / (24 * 60 * 60 * 1000);
        } catch (ParseException e) {
            // TODO 自动生成 catch 块
            e.printStackTrace();
        }
        return day;
    }

    /**
     * 修改日期，得到当前
     *
     * @param startDate 开始的日期
     * @param num       修改的天数
     * @return 修改后的日期 {@link Date}
     */
    public static Date getChangeDayOfDate(String startDate, int num, String pattern) {
        Date date = stringToDate(startDate, pattern);
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        ca.add(Calendar.DAY_OF_YEAR, num);
        //最后一天格式化
        return ca.getTime();
    }

    /**
     * 得到相减的天数(精确到时分秒)
     *
     * @param beginDateStr 开始时间
     * @param endDateStr   截止时间
     * @return 天数
     */
    public static long getDaySubByYMDMS(String beginDateStr, String endDateStr) {
        long day = 0;
        java.text.SimpleDateFormat format = new java.text.SimpleDateFormat(DADE_FROMAT_YMDHMS);
        java.util.Date beginDate;
        java.util.Date endDate;
        try {
            beginDate = format.parse(beginDateStr);
            endDate = format.parse(endDateStr);
            day = (endDate.getTime() - beginDate.getTime()) / (24 * 60 * 60 * 1000);
        } catch (ParseException e) {
            // TODO 自动生成 catch 块
            e.printStackTrace();
        }
        return day;
    }



    public static boolean isValidFormat(String format, String value) {
        Date date = null;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            date = sdf.parse(value);
            if (!value.equals(sdf.format(date))) {
                date = null;
            }
        } catch (ParseException ex) {
            ex.printStackTrace();
        }
        return date != null;
    }

    /**
     * <p>Checks if two calendars represent the same day ignoring time.</p>
     *
     * @param cal1 the first calendar, not altered, not null
     * @param cal2 the second calendar, not altered, not null
     * @return true if they represent the same day
     * @throws IllegalArgumentException if either calendar is <code>null</code>
     */
    public static boolean isSameDay(Calendar cal1, Calendar cal2) {
        if (cal1 == null || cal2 == null) {
            throw new IllegalArgumentException("The dates must not be null");
        }
        return (cal1.get(Calendar.ERA) == cal2.get(Calendar.ERA) &&
                cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
                cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR));
    }

    /**
     * <p>Checks if two dates are on the same day ignoring time.</p>
     *
     * @param date1 the first date, not altered, not null
     * @param date2 the second date, not altered, not null
     * @return true if they represent the same day
     * @throws IllegalArgumentException if either date is <code>null</code>
     */
    public static boolean isSameDay(Date date1, Date date2) throws IllegalArgumentException {
        if (date1 == null || date2 == null) {
            throw new IllegalArgumentException("The dates must not be null");
        }
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        return isSameDay(cal1, cal2);
    }

    /**
     * <p>Checks if a date is today.</p>
     *
     * @param date the date, not altered, not null.
     * @return true if the date is today.
     * @throws IllegalArgumentException if the date is <code>null</code>
     */
    public static boolean isToday(Date date) throws IllegalArgumentException {
        return isSameDay(date, Calendar.getInstance().getTime());
    }

    /**
     * @param weekday from {@link Calendar#SUNDAY} to {@link Calendar#SATURDAY}
     * @return
     */
    public static String getWeekdayStr(int weekday) {
        switch (weekday - 1) {
            case 0:
                return "周日";
            case 1:
                return "周一";
            case 2:
                return "周二";
            case 3:
                return "周三";
            case 4:
                return "周四";
            case 5:
                return "周五";
            case 6:
                return "周六";
            default:
                return "";
        }
    }


    /**
     * 计算当前时间和指定时间之间相差的天数
     *
     * @param date 指定的时间
     */
    public static int daysPassed(Date date) throws NumberFormatException {
        Date start = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(start);
        long time1 = cal.getTimeInMillis();
        cal.setTime(date);
        long time2 = cal.getTimeInMillis();
        long between_days = (time1 - time2) / (1000 * 3600 * 24);
        return Integer.parseInt(String.valueOf(between_days));
    }

    /**
     * 计算当前时间和指定时间之间相差的分钟数
     *
     * @param date 指定的时间
     */
    public static int minutesPassed(Date date) throws NumberFormatException {
        Date start = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(start);
        long time1 = cal.getTimeInMillis();
        cal.setTime(date);
        long time2 = cal.getTimeInMillis();
        long between_days = (time1 - time2) / (1000 * 60);
        return Integer.parseInt(String.valueOf(between_days));
    }

    /**
     * 将日期类型 LocalDate 转换成 Date 类型
     *
     * @param localDate 日期类型 LocalDate
     * @return 日期类型 Date
     */
    public static Date convertToDate(LocalDate localDate) {
        return Date.from(localDate.atStartOfDay(ZoneId.systemDefault()).toInstant());
    }

    /**
     * 将日期类型 LocalDateTime 转换成 Date 类型
     *
     * @param localDateTime 日期类型 LocalDateTime
     * @return 日期类型 Date
     */
    public static Date convertToDate(LocalDateTime localDateTime) {
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * 将日期类型 Date 转换成 LocalDateTime 类型
     *
     * @param date 日期类型 Date
     * @return 日期类型 LocalDateTime
     */
    public static LocalDateTime convertToLocalDateTime(Date date) {
        return date.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    /**
     * 将时间戮转换成 LocalDateTime 类型
     *
     * @param timestamp 时间戮
     * @return 日期类型 LocalDateTime
     */
    public static LocalDateTime convertToLocalDateTime(long timestamp) {
        return Instant.ofEpochMilli(timestamp).atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    /**
     * 将日期类型 LocalDateTime 转换成时间戮
     *
     * @param localDateTime 日期类型 LocalDateTime
     * @return 时间戮
     */
    public static long convertToTimestamp(LocalDateTime localDateTime) {
        return localDateTime.toInstant(ZoneOffset.ofHours(8)).toEpochMilli();
    }

    /**
     * 将日期类型 LocalDate 转换成时间戮
     *
     * @param localDate 日期类型 LocalDate
     * @return 时间戮
     */
    public static long convertToTimestamp(LocalDate localDate) {
        return localDate.atStartOfDay(ZoneId.systemDefault()).toInstant().toEpochMilli();
    }

    /**
     * 将时间戳转换成日期类型
     *
     * @param timestamp 时间戳
     * @return
     */
    public static Date convertTimestampToDate(long timestamp) {
        return new Date(timestamp);
    }
    public static long convertToTimestamp(Date localDate) {
        if (Checker.isNone(localDate)){
            return 0L;
        }
        return localDate.getTime();
    }
}
