package com.ywt.biz.common.util;

import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

public class PinYinUtil {
    public static String getOneFirstSpell(String chinese) {
        String result = getFirstSpell(chinese);
        if (result.length() > 0) {
            result = result.substring(0, 1);
        } else {
            result = "Z";
        }
        return result.toUpperCase();
    }

    /**
     * 获取汉字串拼音首字母，英文字符不变
     * @param chineseCharacts 汉字串
     * @return 汉语拼音首字母
     */
    public static String getFirstSpell(String chineseCharacts) {
        String result = "";//如果没有查到，默认返回Z
        if (null == chineseCharacts) {
            return "";
        }
        chineseCharacts = chineseCharacts.trim();

        try {
            StringBuffer pybf = new StringBuffer();
            char[] arr = chineseCharacts.toCharArray();
            HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
            defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
            defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
            for (int i = 0; i < arr.length; i++) {
                if (arr[i] > 128) {
                    try {
                        String[] temp = PinyinHelper.toHanyuPinyinStringArray(arr[i], defaultFormat);
                        if (temp != null) {
                            pybf.append(temp[0].charAt(0));
                        }
                    } catch (BadHanyuPinyinOutputFormatCombination e) {
                        e.printStackTrace();
                    }
                } else {
                    pybf.append(arr[i]);
                }
            }
            result = pybf.toString().replaceAll("\\W", "");
            result = null == result ? "" : result.trim();
        } catch (Exception ex) {
        }
        return result;
    }

    /**
     * 中文转为拼音
     * @param chineseCharacts
     * @return
     */
    public static String getPingYin(String chineseCharacts) {
        if (null == chineseCharacts || "".equals(chineseCharacts)) {
            return "";
        }
        chineseCharacts = chineseCharacts.trim();

        HanyuPinyinOutputFormat format = new HanyuPinyinOutputFormat();
        format.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        format.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        format.setVCharType(HanyuPinyinVCharType.WITH_V);

        char[] input = chineseCharacts.trim().toCharArray();
        String output = "";

        try {
            for (int i = 0; i < input.length; i++) {
                if (Character.toString(input[i]).matches("[\\u4E00-\\u9FA5]+")) {
                    String[] temp = PinyinHelper.toHanyuPinyinStringArray(input[i], format);
                    output += temp[0];
                } else
                    output += Character.toString(input[i]);
            }
        } catch (BadHanyuPinyinOutputFormatCombination e) {
            e.printStackTrace();
        }
        return output;
    }

    /**
     * 获取汉字串拼音，英文字符不变
     * @param chineseCharacts 汉字串
     * @return 汉语拼音
     */
    public static String getFullSpell(String chineseCharacts) {
        StringBuffer pybf = new StringBuffer();
        char[] arr = chineseCharacts.toCharArray();
        HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
        defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        for (int i = 0; i < arr.length; i++) {
            if (arr[i] > 128) {
                try {
                    pybf.append(PinyinHelper.toHanyuPinyinStringArray(arr[i], defaultFormat)[0]);
                } catch (BadHanyuPinyinOutputFormatCombination e) {
                    e.printStackTrace();
                }
            } else {
                pybf.append(arr[i]);
            }
        }
        return pybf.toString();
    }
}
